import type { Credentials, OcJsonLdDocument } from "@/declarations";
import { httpFetch } from "@/helpers/http";
import { compact, frame, type ContextDefinition } from "jsonld";
import type { Frame } from "jsonld/jsonld-spec";

/**
 * Execute a CONTRUCT SparQL query and return
 * a result compacted with the help of
 *
 * - the given context
 * - the context available in the server response if no
 * context if given.
 */
export async function executeSparqlConstruct<T>(
  constructQuery: string,
  options?: {
    auth?: Credentials,
    context?: ContextDefinition,
    frame?: Frame
  }
): Promise<OcJsonLdDocument<T>> {
  const baseUrl = `${import.meta.env.VITE_OC_BASE_URL}/${options?.auth ? 'sparql-auth' : 'sparql'}`
  const url = baseUrl + '?' + new URLSearchParams({ query: constructQuery }).toString()

  const response = await httpFetch(
    url,
    {
      method: 'GET',
      headers: {
        'Content-Type': "application/x-www-form-urlencoded",
        "Accept": "application/ld+json"
      },
      auth: options?.auth
    }
  )
  const data = JSON.parse(response.data)

  let compacted = await compact(data, options?.context ?? data['@context'])
  if (options?.frame) {
    compacted = await frame(compacted, options.frame)
  }

  // If there's only one result, `compacted['@graph']` does not exist
  // and the only object is at the root of the `compacted` Object.
  // The lines below are here to normalize this.
  const graph: T[] = []
  if (compacted['@graph'] === undefined) {
    const { ['@context']: _, ...data } = compacted;
    graph.push(data as T)
  } else {
    graph.push(...(compacted['@graph'] as Iterable<T>))
  }

  return {
    graph: graph as T[],
    context: compacted['@context'] as ContextDefinition,
  }
}

/**
 * Execute a SparQL Insert query
 */
export async function executeSparqlInsert(
  insertQuery: string,
  options?: {
    auth?: Credentials,
    asVisitor?: boolean,
  }
) {
  const baseUrl = `${import.meta.env.VITE_OC_BASE_URL}/sparql-auth`
  const url = baseUrl + '?' + new URLSearchParams({ query: insertQuery }).toString()

  if (options?.auth === undefined && options?.asVisitor === undefined) {
    throw new Error('Insert SparQL query should be launched with an authentification or as the visitor user.')
  }

  return await httpFetch(
    url,
    {
      method: 'GET',
      headers: {
        'Content-Type': "application/x-www-form-urlencoded",
        "Accept": "application/sparql-results+xml"
      },
      auth: options?.auth ?? {
        username: import.meta.env.VITE_VISITOR_USER,
        password: import.meta.env.VITE_VISITOR_PWD,
      }
    }
  )
}
